"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildInvalidator = exports.getDomainAndSubdomain = exports.createAliasRecord = exports.buildCDN = exports.uploadStatic = exports.buildStatic = exports.validateCertificate = exports.buildRouter = exports.getLambdaRole = void 0;
const fs = require("fs");
const path = require("path");
const mime = require("mime-types");
const aws = require("@pulumi/aws");
const pulumi = require("@pulumi/pulumi");
const command_1 = require("@pulumi/command");
const utils_js_1 = require("../utils.js");
const nameRegister = utils_js_1.NameRegister.getInstance();
let registerName = (name) => {
    return nameRegister.registerName(name);
};
const eastRegion = new aws.Provider(registerName('ProviderEast'), {
    region: 'us-east-1',
});
function getLambdaRole(functionArns) {
    let lambdaPolicyStub = {
        statements: [
            {
                principals: [
                    {
                        type: 'Service',
                        identifiers: ['lambda.amazonaws.com', 'edgelambda.amazonaws.com'],
                    },
                ],
                actions: ['sts:AssumeRole'],
                effect: 'Allow',
            },
        ],
    };
    let lambdaPolicyDocument = aws.iam.getPolicyDocumentOutput(lambdaPolicyStub);
    const iamForLambda = new aws.iam.Role(registerName('IamForLambda'), {
        assumeRolePolicy: lambdaPolicyDocument.json,
    });
    new aws.iam.RolePolicyAttachment(registerName('ServerRPABasicExecutionRole'), {
        role: iamForLambda.name,
        policyArn: aws.iam.ManagedPolicy.AWSLambdaBasicExecutionRole,
    });
    if (functionArns) {
        lambdaPolicyStub = {
            statements: [
                {
                    actions: ['lambda:InvokeFunctionUrl'],
                    effect: 'Allow',
                    resources: functionArns,
                },
            ],
        };
        lambdaPolicyDocument = aws.iam.getPolicyDocumentOutput(lambdaPolicyStub);
        const policy = new aws.iam.Policy(registerName('invokePolicy'), {
            policy: lambdaPolicyDocument.json,
        });
        new aws.iam.RolePolicyAttachment(registerName('ServerRPAInvokePolicy'), {
            role: iamForLambda.name,
            policyArn: policy.arn,
        });
    }
    return iamForLambda;
}
exports.getLambdaRole = getLambdaRole;
function buildRouter(iamForLambda, routerPath) {
    const routerHandler = new aws.lambda.Function(registerName('LambdaRouterFunctionHandler'), {
        code: new pulumi.asset.FileArchive(routerPath),
        role: iamForLambda.arn,
        handler: 'router.handler',
        runtime: 'nodejs18.x',
        memorySize: 128,
        publish: true,
    }, { provider: eastRegion });
    return routerHandler;
}
exports.buildRouter = buildRouter;
function validateCertificate(FQDN, domainName) {
    if (!FQDN.includes(domainName)) {
        throw new Error('FQDN must contain domainName');
    }
    const certificate = new aws.acm.Certificate(registerName('Certificate'), {
        domainName: FQDN,
        validationMethod: 'DNS',
    }, { provider: eastRegion });
    const hostedZone = aws.route53.getZone({
        name: domainName,
        privateZone: false,
    });
    const validationRecord = new aws.route53.Record(registerName(`${FQDN}.validation`), {
        name: certificate.domainValidationOptions[0].resourceRecordName,
        records: [certificate.domainValidationOptions[0].resourceRecordValue],
        ttl: 60,
        type: certificate.domainValidationOptions[0].resourceRecordType,
        zoneId: hostedZone.then((x) => x.zoneId),
    });
    const certificateValidation = new aws.acm.CertificateValidation(registerName('CertificateValidation'), {
        certificateArn: certificate.arn,
        validationRecordFqdns: [validationRecord.fqdn],
    }, { provider: eastRegion });
    return certificateValidation.certificateArn;
}
exports.validateCertificate = validateCertificate;
function buildStatic(staticPath, prerenderedPath) {
    const bucket = new aws.s3.Bucket(registerName('StaticContentBucket'), {
        acl: 'private',
        forceDestroy: true,
    });
    exports.uploadStatic(staticPath, bucket);
    exports.uploadStatic(prerenderedPath, bucket);
    return bucket;
}
exports.buildStatic = buildStatic;
// Sync the contents of the source directory with the S3 bucket, which will
// in-turn show up on the CDN.
function uploadStatic(dirPath, bucket) {
    // crawlDirectory recursive crawls the provided directory, applying the
    // provided function to every file it contains. Doesn't handle cycles from
    // symlinks.
    function crawlDirectory(dir, f) {
        const files = fs.readdirSync(dir);
        for (const file of files) {
            const filePath = path.join(dir, file);
            const stat = fs.statSync(filePath);
            if (stat.isDirectory()) {
                crawlDirectory(filePath, f);
            }
            if (stat.isFile()) {
                f(filePath);
            }
        }
    }
    console.log('Syncing contents from local disk at', dirPath);
    crawlDirectory(dirPath, (filePath) => {
        const relativeFilePath = filePath.replace(dirPath + path.sep, '');
        const posixFilePath = relativeFilePath.split(path.sep).join(path.posix.sep);
        const contentFile = new aws.s3.BucketObject(registerName(posixFilePath), {
            key: posixFilePath,
            bucket: bucket.id,
            contentType: mime.lookup(filePath) || undefined,
            source: new pulumi.asset.FileAsset(filePath),
        }, {
            parent: bucket,
        });
    });
}
exports.uploadStatic = uploadStatic;
function buildCDN(routerFunction, bucket, serverHeaders, FQDN, certificateArn) {
    const defaultRequestPolicy = new aws.cloudfront.OriginRequestPolicy(registerName('DefaultRequestPolicy'), {
        cookiesConfig: {
            cookieBehavior: 'all',
        },
        headersConfig: {
            headerBehavior: 'whitelist',
            headers: {
                items: serverHeaders,
            },
        },
        queryStringsConfig: {
            queryStringBehavior: 'all',
        },
    });
    const oac = new aws.cloudfront.OriginAccessControl(registerName('CloudFrontOriginAccessControl'), {
        description: 'Default Origin Access Control',
        name: 'CloudFrontOriginAccessControl',
        originAccessControlOriginType: 's3',
        signingBehavior: 'no-override',
        signingProtocol: 'sigv4',
    });
    const optimizedCachePolicy = aws.cloudfront.getCachePolicyOutput({
        name: 'Managed-CachingOptimized',
    });
    const distribution = new aws.cloudfront.Distribution(registerName('CloudFrontDistribution'), {
        enabled: true,
        origins: [
            {
                originId: 'default',
                domainName: bucket.bucketRegionalDomainName,
                originAccessControlId: oac.id,
            },
        ],
        aliases: FQDN ? [FQDN] : undefined,
        priceClass: 'PriceClass_100',
        viewerCertificate: FQDN
            ? {
                // Per AWS, ACM certificate must be in the us-east-1 region.
                acmCertificateArn: certificateArn,
                sslSupportMethod: 'sni-only',
            }
            : {
                cloudfrontDefaultCertificate: true,
            },
        defaultCacheBehavior: {
            compress: true,
            viewerProtocolPolicy: 'redirect-to-https',
            allowedMethods: [
                'DELETE',
                'GET',
                'HEAD',
                'OPTIONS',
                'PATCH',
                'POST',
                'PUT',
            ],
            cachedMethods: ['GET', 'HEAD', 'OPTIONS'],
            lambdaFunctionAssociations: [
                {
                    eventType: 'origin-request',
                    lambdaArn: pulumi
                        .all([routerFunction.arn, routerFunction.version])
                        .apply(([arn, version]) => {
                        return `${arn}:${version}`;
                    }),
                    includeBody: true,
                },
            ],
            originRequestPolicyId: defaultRequestPolicy.id,
            cachePolicyId: optimizedCachePolicy.apply((policy) => policy.id),
            targetOriginId: 'default',
        },
        restrictions: {
            geoRestriction: {
                restrictionType: 'none',
            },
        },
    });
    const cloudFrontPolicyDocument = aws.iam.getPolicyDocumentOutput({
        statements: [
            {
                principals: [
                    {
                        type: 'Service',
                        identifiers: ['cloudfront.amazonaws.com'],
                    },
                ],
                actions: ['s3:GetObject'],
                resources: [pulumi.interpolate `${bucket.arn}/\*`],
                conditions: [
                    {
                        test: 'StringEquals',
                        variable: 'AWS:SourceArn',
                        values: [distribution.arn],
                    },
                ],
            },
            {
                principals: [
                    {
                        type: 'AWS',
                        identifiers: ['*'],
                    },
                ],
                actions: ['s3:*'],
                effect: 'Deny',
                resources: [pulumi.interpolate `${bucket.arn}/\*`, bucket.arn],
                conditions: [
                    {
                        test: 'Bool',
                        variable: 'aws:SecureTransport',
                        values: ['false'],
                    },
                ],
            },
        ],
    });
    const cloudFrontBucketPolicy = new aws.s3.BucketPolicy(registerName('CloudFrontBucketPolicy'), {
        bucket: bucket.id,
        policy: cloudFrontPolicyDocument.apply((policy) => policy.json),
    });
    return distribution;
}
exports.buildCDN = buildCDN;
// Creates a new Route53 DNS record pointing the domain to the CloudFront
// distribution.
function createAliasRecord(targetDomain, distribution) {
    const domainParts = exports.getDomainAndSubdomain(targetDomain);
    const hostedZoneId = aws.route53
        .getZone({ name: domainParts.parentDomain }, { async: true })
        .then((zone) => zone.zoneId);
    return new aws.route53.Record(registerName(targetDomain), {
        name: domainParts.subdomain,
        zoneId: hostedZoneId,
        type: 'A',
        aliases: [
            {
                name: distribution.domainName,
                zoneId: distribution.hostedZoneId,
                evaluateTargetHealth: true,
            },
        ],
    });
}
exports.createAliasRecord = createAliasRecord;
// Split a domain name into its subdomain and parent domain names.
// e.g. "www.example.com" => "www", "example.com".
function getDomainAndSubdomain(domain) {
    const parts = domain.split('.');
    if (parts.length < 2) {
        throw new Error(`No TLD found on ${domain}`);
    }
    // No subdomain, e.g. awesome-website.com.
    if (parts.length === 2) {
        return { subdomain: '', parentDomain: domain };
    }
    const subdomain = parts[0];
    parts.shift(); // Drop first element.
    return {
        subdomain,
        // Trailing "." to canonicalize domain.
        parentDomain: parts.join('.') + '.',
    };
}
exports.getDomainAndSubdomain = getDomainAndSubdomain;
function buildInvalidator(distribution, staticPath, prerenderedPath) {
    const pathHashProvider = {
        create(inputs) {
            return __awaiter(this, void 0, void 0, function* () {
                const folderHash = yield Promise.resolve().then(() => require('folder-hash'));
                const pathHash = yield folderHash.hashElement(inputs.path);
                return { id: inputs.path, outs: { hash: pathHash.toString() } };
            });
        },
        diff(id, previousOutput, news) {
            return __awaiter(this, void 0, void 0, function* () {
                const replaces = [];
                let changes = true;
                const oldHash = previousOutput.hash;
                const folderHash = yield Promise.resolve().then(() => require('folder-hash'));
                const newHash = yield folderHash.hashElement(news.path);
                if (oldHash === newHash.toString()) {
                    changes = false;
                }
                return {
                    deleteBeforeReplace: false,
                    replaces: replaces,
                    changes: changes,
                };
            });
        },
        update(id, olds, news) {
            return __awaiter(this, void 0, void 0, function* () {
                const folderHash = yield Promise.resolve().then(() => require('folder-hash'));
                const pathHash = yield folderHash.hashElement(news.path);
                return { outs: { hash: pathHash.toString() } };
            });
        },
    };
    class PathHash extends pulumi.dynamic.Resource {
        constructor(name, args, opts) {
            super(pathHashProvider, name, Object.assign({ hash: undefined }, args), opts);
        }
    }
    let staticHash = new PathHash(registerName('StaticHash'), {
        path: staticPath,
    });
    let prerenderedHash = new PathHash(registerName('PrerenderedHash'), {
        path: prerenderedPath,
    });
    const invalidationCommand = new command_1.local.Command(registerName('Invalidate'), {
        create: pulumi.interpolate `aws cloudfront create-invalidation --distribution-id ${distribution.id} --paths /\*`,
        triggers: [staticHash.hash, prerenderedHash.hash],
    }, {
        dependsOn: [distribution],
    });
}
exports.buildInvalidator = buildInvalidator;
//# sourceMappingURL=resources.js.map